@@ -9,7 +9,7 @@ from pxr.Usdviewq import plugin, layerStackContextMenu, attributeViewContextMenu
 
 from ._qt import QtWidgets
 
-from . import _core, _attributes, sheets as _sheets, description as _description, create as _create
+from . import _core, _attributes, sheets as _sheets, description as _description, create as _create, stats as _stats
 
 _usdview_api = contextvars.ContextVar("_usdview_api")  # TODO: is there a better way?
 _description._PALETTE.set(0)  # TODO 2: same question (0 == dark, 1 == light)
@@ -36,7 +36,7 @@ def _layer_stack_from_prims(usdviewApi):
 @lru_cache(maxsize=None)
 def prim_composition(usdviewApi):
     widget = _description.PrimComposition(parent=usdviewApi.qMainWindow)
-
+    widget.setStyleSheet(_core._USDVIEW_QTREEVIEW_STYLE)
     def primChanged(new_paths, __):
         new_path = next(iter(new_paths), None)
         widget.setPrim(usdviewApi.stage.GetPrimAtPath(new_path)) if new_path else widget.clear()
@@ -73,13 +73,11 @@ class GrillContentBrowserLayerMenuItem(layerStackContextMenu.LayerStackContextMe
         if self._item:
             usdview_api = _usdview_api.get()
             context = usdview_api.stage.GetPathResolverContext()
-            layer = getattr(self._item, 'layer', None)  # USDView allows for single layer selection in composition tab :(
-            if not layer:
+            if not (layer:= getattr(self._item, 'layer', None)):  # USDView allows for single layer selection in composition tab :(
                 layerPath = getattr(self._item, 'layerPath', "")
                 # We're protected by the IsEnabled method above, so don't bother checkin layerPath value
                 with Ar.ResolverContextBinder(context):
-                    layer = Sdf.Layer.FindOrOpen(layerPath)
-                    if not layer:  # edge case, is this possible?
+                    if not (layer:=Sdf.Layer.FindOrOpen(layerPath)):  # edge case, is this possible?
                         print(f"Could not find layer from {layerPath}")
                         return
             _description._launch_content_browser([layer], usdview_api.qMainWindow, context)
@@ -97,11 +95,10 @@ class GrillAttributeEditorMenuItem(attributeViewContextMenu.AttributeViewContext
         return self._item and self._attributes
 
     def GetText(self):
-        return "Edit Value (s)"
+        return f"Edit Value{'s' if len(self._attributes)>1 else ''}"
 
     def RunCommand(self):
-        attributes = self._attributes
-        if attributes:
+        if attributes:=self._attributes:
             editor = _ValueEditor(_usdview_api.get().qMainWindow)
             editor.setAttributes(attributes)
             editor.show()
@@ -118,8 +115,8 @@ class _ValueEditor(QtWidgets.QDialog):
         layout = self.layout()
         supported_primvars = {"displayColor"}
         for attr in attributes:
-            primvar = UsdGeom.Primvar(attr)
-            if primvar and primvar.GetPrimvarName() in supported_primvars:
+            print(attr)
+            if (primvar:= UsdGeom.Primvar(attr)) and primvar.GetPrimvarName() in supported_primvars:
                 editor = _attributes._DisplayColorEditor(primvar)
                 layout.addRow(primvar.GetPrimvarName(), editor)
             elif attr.GetTypeName() == Sdf.ValueTypeNames.Double:
@@ -155,8 +152,7 @@ class GrillPlugin(plugin.PluginContainer):
             )
 
         self._menu_items = [
-            *(_menu_item(title, launcher)
-            for (title, launcher) in (
+            *(_menu_item(title, launcher) for (title, launcher) in (
                 ("Create Assets", _stage_on_widget(_create.CreateAssets)),
                 ("Taxonomy Editor", _stage_on_widget(_create.TaxonomyEditor)),
                 ("Spreadsheet Editor", _stage_on_widget(_sheets.SpreadsheetEditor)),
@@ -166,7 +162,11 @@ class GrillPlugin(plugin.PluginContainer):
                 _menu_item("From Current Stage", _stage_on_widget(_description.LayerStackComposition)),
                 _menu_item("From Selected Prims", _layer_stack_from_prims),
             ]},
-            _menu_item("Save Changes", save_changes),
+            operator.methodcaller("addSeparator"),
+            *(_menu_item(title, launcher) for title, launcher in (
+                ("Stage Stats", _stage_on_widget(_stats.StageStats)),
+                ("Save Changes", save_changes),
+            )),
             operator.methodcaller("addSeparator"),
             {"Preferences": [_menu_item("Repository Path", repository_path)],},
         ]
@@ -186,14 +186,14 @@ class GrillPlugin(plugin.PluginContainer):
         _populate_menu(grill_menu, self._menu_items)
 
 
+def _extend_menu(_extender, original, *args):
+    return [_extender(*args)] + original(*args)  # if it looks like a duck
+
+
 for module, member_name, extender in (
         (layerStackContextMenu, "_GetContextMenuItems", GrillContentBrowserLayerMenuItem),
-        # _GetContextMenuItems(item, dataModel) signature is inverse than GrillAttributeEditorMenuItem(dataModel, item)
-        (attributeViewContextMenu, "_GetContextMenuItems", lambda *args: GrillAttributeEditorMenuItem(*reversed(args)))
+        (attributeViewContextMenu, "_GetContextMenuItems", lambda *args: GrillAttributeEditorMenuItem(*reversed(args)))  # _GetContextMenuItems(item, dataModel) signature is inverse than GrillAttributeEditorMenuItem(dataModel, item)
 ):
-    def _extended(_extender, original, *args):
-        return [_extender(*args)] + original(*args)  # if it looks like a duck
-    setattr(module, member_name, partial(_extended, extender, getattr(module, member_name)))
-
+    setattr(module, member_name, partial(_extend_menu, extender, getattr(module, member_name)))
 
 Tf.Type.Define(GrillPlugin)
